﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PrintingTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public List<string> NameList
        {
            get;
            set;
        }

        public MainWindow()
        {
            InitializeComponent();
            InitilizeList();
            NameList1.ItemsSource = NameList;
            NameList2.ItemsSource = NameList;
            NameList3.ItemsSource = NameList;
        }

        private void InitilizeList()
        {
            NameList = new List<string>();

            for (int i = 0; i < 30; i++)
            {
                NameList.Add("ListItem" + i);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Forms.PrintDialog dialog = new System.Windows.Forms.PrintDialog();

            dialog.Document = this.GetPrintDocument();
            dialog.AllowSomePages = true;
            dialog.AllowCurrentPage = true;

            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                dialog.Document.Print();
            }     
        }

        public System.Drawing.Printing.PrintDocument GetPrintDocument()
        {
            System.Drawing.Printing.PrintDocument printDocument = new System.Drawing.Printing.PrintDocument();
            printDocument.DefaultPageSettings.Landscape = false; // Set orientation here

            printDocument.PrinterSettings.MaximumPage = 1;// set maximum page count here
            bool isCalculatedrange = false;
            int printedpagescount = 0;
            int startPage = 0;
            int printPageCount = 0;


            printDocument.PrintPage += (s, args) =>
            {
                try
                {
                    if (printDocument.PrinterSettings.ToPage > 0 && !isCalculatedrange)
                    {
                        startPage = printDocument.PrinterSettings.FromPage - 1;
                        printPageCount = (printDocument.PrinterSettings.ToPage - printDocument.PrinterSettings.FromPage) + 1 > printPageCount ? printPageCount : (printDocument.PrinterSettings.ToPage - printDocument.PrinterSettings.FromPage) + 1;
                        isCalculatedrange = true;
                    }

                    args.Graphics.DrawImage(GetImageFromUIElement(grd_Page, 96, 96), 
                        new System.Drawing.Rectangle(printDocument.PrinterSettings.DefaultPageSettings.Margins.Left, printDocument.PrinterSettings.DefaultPageSettings.Margins.Top, (int)grd_Page.ActualWidth, (int)grd_Page.ActualHeight));
                    startPage++;
                    printedpagescount++;
                    args.HasMorePages = startPage < printPageCount;

                    if (isCalculatedrange)
                    {
                        args.HasMorePages = (printPageCount != printedpagescount);
                    }

                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message + "\n StackTrace: \n" + e.StackTrace);
                }
            };

            printDocument.EndPrint += (s, args) =>
            {
                MessageBox.Show("Documents printed completed...");
            };

            return printDocument;
        }

        public static System.Drawing.Image GetImageFromUIElement(Grid source, int dpix, int dpiy)
        {
            int width = (int)source.ActualWidth;
            int height = (int)source.ActualHeight;

            if (width == 0)
                width = (int)source.Width;
            if (height == 0)
                height = (int)source.Height;

            int dpiwidth = (int)((width * dpix) / 96);
            int dpiheight = (int)((height * dpiy) / 96);

            RenderTargetBitmap bitmap = new RenderTargetBitmap(dpiwidth, dpiheight, dpix, dpiy, PixelFormats.Pbgra32);
            bitmap.Render(source);

            var encoder = new PngBitmapEncoder();

            encoder.Frames.Add(BitmapFrame.Create(bitmap));

            using (var stream = new MemoryStream())
            {
                encoder.Save(stream);
                source.Clip = null;
                bitmap.Freeze();
                bitmap.Clear();
                return System.Drawing.Image.FromStream(stream);
            }
        }

    }
}
